==> fred-backend-registry_2.0.0.orig.tar.gz: fred-backend-registry-2.0.0/test/main.cc <==
/*
 * Copyright (C) 2018-2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE TestBackendRegistry

#include "config.h"

#include "test/setup/fixture/cfg.hh"
#include "test/setup/fixture/contact.hh"
#include "test/setup/fixture/registrar.hh"

#include "libfred/db_settings.hh"
#include "libfred/registrable_object/contact/create_contact.hh"
#include "libfred/registrar/get_registrar_handles.hh"

#include "util/db/manager.hh"
#include "util/log/logger.hh"

// dynamic library version
#include <boost/test/unit_test.hpp>
#include <boost/assign/list_of.hpp>

#include <cstdlib>
#include <iostream>

namespace {

enum class LogHandler
{
    is_set,
    is_not_set
};

class SetLogHandler : public boost::static_visitor<LogHandler>
{
public:
    explicit SetLogHandler(Logging::Log& logger_instance) : logger_instance_(logger_instance) { }
    LogHandler operator()(const boost::blank&)const
    {
        return LogHandler::is_not_set;
    }
    LogHandler operator()(const Test::Setup::Fixture::Cfg::Options::Console& option)const
    {
        logger_instance_.add_handler_of<Logging::Log::Device::console>(option.min_severity);
        return LogHandler::is_set;
    }
    LogHandler operator()(const Test::Setup::Fixture::Cfg::Options::LogFile& option)const
    {
        logger_instance_.add_handler_of<Logging::Log::Device::file>(option.file_name, option.min_severity);
        return LogHandler::is_set;
    }
    LogHandler operator()(const Test::Setup::Fixture::Cfg::Options::SysLog& option)const
    {
        logger_instance_.add_handler_of<Logging::Log::Device::syslog>(option.facility, option.min_severity);
        return LogHandler::is_set;
    }
private:
    Logging::Log& logger_instance_;
};

std::string make_connect_string(const Test::Setup::Fixture::Cfg::Options::Database& option)
{
    std::string result;
    if (option.host != boost::none)
    {
        result = "host=" + *option.host;
    }
    if (option.port != boost::none)
    {
        if (!result.empty())
        {
            result += " ";
        }
        result += "port=" + std::to_string(*option.port);
    }
    if (option.user != boost::none)
    {
        if (!result.empty())
        {
            result += " ";
        }
        result += "user=" + *option.user;
    }
    if (option.dbname != boost::none)
    {
        if (!result.empty())
        {
            result += " ";
        }
        result += "dbname=" + *option.dbname;
    }
    if (option.password != boost::none)
    {
        if (!result.empty())
        {
            result += " ";
        }
        result += "password=" + *option.password;
    }
    return result;
}

struct GlobalConfigFixture
{
    GlobalConfigFixture()
    {
        try
        {
            Test::Setup::Fixture::Cfg::Options::init(boost::unit_test::framework::master_test_suite().argc,
                                                     boost::unit_test::framework::master_test_suite().argv);
        }
        catch (const Test::Setup::Fixture::Cfg::HelpRequested& help)
        {
            std::cout << help.what() << std::endl;
            std::exit(EXIT_SUCCESS);
        }
        catch (const Test::Setup::Fixture::Cfg::UnknownOption& unknown_option)
        {
            std::cerr << unknown_option.what() << std::endl;
            std::exit(EXIT_FAILURE);
        }
        catch (const Test::Setup::Fixture::Cfg::MissingOption& missing_option)
        {
            std::cerr << missing_option.what() << std::endl;
            std::exit(EXIT_FAILURE);
        }
        catch (const Test::Setup::Fixture::Cfg::Exception& e)
        {
            std::cerr << "Configuration problem: " << e.what() << std::endl;
            std::exit(EXIT_FAILURE);
        }
        catch (const std::exception& e)
        {
            std::cerr << "Exception caught: " << e.what() << std::endl;
            std::exit(EXIT_FAILURE);
        }
        catch (...)
        {
            std::cerr << "Unexpected exception caught" << std::endl;
            std::exit(EXIT_FAILURE);
        }

        switch (boost::apply_visitor(SetLogHandler(LOGGER), Test::Setup::Fixture::Cfg::Options::get().log))
        {
            case LogHandler::is_set:
                LOGGER.debug("test-fred-registry-services configuration successfully loaded");
                break;
            case LogHandler::is_not_set:
                std::cout << "test-fred-registry-services configuration successfully loaded" << std::endl;
                break;
        }
    }
};

class GlobalAdminDatabaseFixture : public GlobalConfigFixture
{
public:
    GlobalAdminDatabaseFixture()
        : original_db_name_(check_dbname(*Test::Setup::Fixture::Cfg::Options::get().database.dbname)),
          temporary_copy_db_name_(check_dbname(original_db_name_ + "_tmp_copy")),
          result_db_name_(check_dbname(original_db_name_ + "_test_result")),
          db_connect_string_(make_connect_string(Test::Setup::Fixture::Cfg::Options::get().admin_database)),
          db_user_(*Test::Setup::Fixture::Cfg::Options::get().database.user)
    {
        LOGGER.debug("GlobalAdminDatabaseFixture()"
                ":original_db_name_(" + original_db_name_ + "),"
                 "temporary_copy_db_name_(" + temporary_copy_db_name_ + "),"
                 "result_db_name_(" + result_db_name_ + "),"
                 "db_connect_string_(" + db_connect_string_ + "),"
                 "db_user_(" + db_user_ + ")");
        const auto db_conn = this->get_admin_connection();
        force_copy_db(db_conn.get(), original_db_name_, temporary_copy_db_name_, db_user_);
    }
    ~GlobalAdminDatabaseFixture()
    {
        try
        {
            const auto db_conn = this->get_admin_connection();
            noexcept_force_copy_db(db_conn.get(), original_db_name_, result_db_name_, db_user_);
            noexcept_force_copy_db(db_conn.get(), temporary_copy_db_name_, original_db_name_, db_user_);
            noexcept_force_drop_db(db_conn.get(), temporary_copy_db_name_, db_user_);
            noexcept_enable_connections(db_conn.get(), result_db_name_);
            noexcept_enable_connections(db_conn.get(), original_db_name_);
            noexcept_enable_connections(db_conn.get(), temporary_copy_db_name_);
        }
        catch (...)
        {
            try
            {
                LOGGER.error("get_admin_connection() failed");
            }
            catch (...)
            {
            }
        }
    }
private:
    std::unique_ptr<Database::StandaloneConnection> get_admin_connection()const
    {
        return Database::StandaloneManager(db_connect_string_).acquire();
    }
    static void disable_connections_add_terminate_persistent_connections(
            Database::StandaloneConnection* const conn,
            const std::string& db_name,
            const std::string& db_user)
    {
        conn->exec_params("UPDATE pg_database SET datallowconn=false WHERE datname=$1::TEXT",
                          Database::query_param_list(db_name));

        std::string pid_column_name;
        {
            const Database::Result dbres = conn->exec(
                    "SELECT column_name "
                    "FROM information_schema.columns "
                    "WHERE table_name='pg_stat_activity' AND "
                          "column_name IN ('procpid','pid')");
            if (dbres.size() <= 0)
            {
                std::runtime_error("table 'pg_stat_activity' contains neither procpid nor pid column");
            }
            pid_column_name = static_cast<std::string>(dbres[0][0]);
        }
        const std::string query =
                "SELECT pg_terminate_backend(" + pid_column_name + ") "
                "FROM pg_stat_activity "
                "WHERE (datname=$1::TEXT OR usename=$2::TEXT) AND " + pid_column_name + "!=pg_backend_pid()";
        const auto params = Database::query_param_list(db_name)(db_user);
        while (conn->exec_params(query, params).size() != 0)
        { }
    }
    static void noexcept_enable_connections(
            Database::StandaloneConnection* const conn,
            const std::string& db_name)noexcept
    {
        try
        {
            conn->exec_params("UPDATE pg_database SET datallowconn=true WHERE datname=$1::TEXT",
                              Database::query_param_list(db_name));
        }
        catch (...)
        {
            try
            {
                LOGGER.error("UPDATE pg_database failed");
            }
            catch (...)
            {
            }
        }
    }
    static const std::string& check_dbname(const std::string& db_name)
    {
        static constexpr unsigned max_postgresql_database_name_length = 63;
        if (max_postgresql_database_name_length < db_name.length())
        {
            throw std::runtime_error(
                    "db_name.length(): " + std::to_string(db_name.length()) + " > "
                    "max_postgresql_database_name_length: " + std::to_string(max_postgresql_database_name_length) + " "
                    "db_name: " + db_name);
        }
        return db_name;
    }
    static void force_drop_db(
            Database::StandaloneConnection* const conn,
            const std::string& db_name,
            const std::string& db_user)
    {
        disable_connections_add_terminate_persistent_connections(conn, db_name, db_user);
        conn->exec("DROP DATABASE IF EXISTS \"" + db_name + "\"");
    }
    static void force_copy_db(
            Database::StandaloneConnection* const conn,
            const std::string& src_name,
            const std::string& dst_name,
            const std::string& db_user)
    {
        force_drop_db(conn, dst_name, db_user);
        conn->exec("CREATE DATABASE \"" + dst_name + "\" TEMPLATE \"" + src_name + "\"");
    }
    static void noexcept_force_drop_db(
            Database::StandaloneConnection* const conn,
            const std::string& db_name,
            const std::string& db_user)noexcept
    {
        try
        {
            disable_connections_add_terminate_persistent_connections(conn, db_name, db_user);
        }
        catch (...)
        {
            try
            {
                LOGGER.error("disable_connections_add_terminate_persistent_connections failed");
            }
            catch (...)
            {
            }
        }
        try
        {
            conn->exec("DROP DATABASE IF EXISTS \"" + db_name + "\"");
        }
        catch (...)
        {
            try
            {
                LOGGER.error("DROP DATABASE failed");
            }
            catch (...)
            {
            }
        }
    }
    static void noexcept_force_copy_db(
            Database::StandaloneConnection* const conn,
            const std::string& src_name,
            const std::string& dst_name,
            const std::string& db_user)
    {
        noexcept_force_drop_db(conn, dst_name, db_user);
        try
        {
            conn->exec("CREATE DATABASE \"" + dst_name + "\" TEMPLATE \"" + src_name + "\"");
        }
        catch (...)
        {
            try
            {
                LOGGER.error("CREATE DATABASE failed");
            }
            catch (...)
            {
            }
        }
    }
    const std::string original_db_name_;
    const std::string temporary_copy_db_name_;
    const std::string result_db_name_;
    const std::string db_connect_string_;
    const std::string db_user_;
};

struct GlobalDatabaseFixture : GlobalAdminDatabaseFixture
{
    GlobalDatabaseFixture()
    {
        Database::emplace_default_manager<Database::StandaloneManager>(
                make_connect_string(Test::Setup::Fixture::Cfg::Options::get().database));
    }
};

struct GlobalRegistryDataFixture : GlobalDatabaseFixture
{
    GlobalRegistryDataFixture()
    {
        LibFred::OperationContextCreator ctx;
        Test::Registrar::init<Test::Registrar::system>(ctx);
        LibFred::Contact::PlaceAddress place;
        place.street1 = "Kontaktní 1";
        place.city = "Praha 1";
        place.postalcode = "111 50";
        place.country = "CZ";
        LibFred::ContactAddressList addresses_by_type;
        LibFred::ContactAddress address;
        address.street1 = "Korespondenční 1";
        address.city = "Praha 1";
        address.postalcode = "111 50";
        address.country = "CZ";
        addresses_by_type[LibFred::ContactAddressType::MAILING] = address;
        address.street1 = "Fakturační 1";
        addresses_by_type[LibFred::ContactAddressType::BILLING] = address;
        address.street1 = "Doručovací 1";
        addresses_by_type[LibFred::ContactAddressType::SHIPPING] = address;
        address.street1 = "Doručovací 2";
        addresses_by_type[LibFred::ContactAddressType::SHIPPING_2] = address;
        address.street1 = "Doručovací 3";
        addresses_by_type[LibFred::ContactAddressType::SHIPPING_3] = address;
        Test::Contact::init(
                ctx,
                Test::Contact::make_handle("TEST_KONTAKT_0001"),
                Test::Registrar::get<Test::Registrar::system>().get_handle(),
                Test::Contact::make_name("Test Kontakt"),
                Test::Contact::make_organization("První Testovací a.s."),
                Test::Contact::make_authinfo("AbCdEfGhIjK1"),
                Test::Contact::make_place_address(place),
                Test::Contact::make_contact_address_list(addresses_by_type),
                Test::Contact::Disclose::name(),
                Test::Contact::Disclose::organization(),
                Test::Contact::Disclose::address(),
                Test::Contact::Hide::telephone(),
                Test::Contact::Hide::fax(),
                Test::Contact::Hide::email(),
                Test::Contact::Hide::notify_email(),
                Test::Contact::Hide::vat_identification_number(),
                Test::Contact::Hide::ident());
        ctx.commit_transaction();
    }
};

struct GlobalFixture : GlobalRegistryDataFixture
{
    GlobalFixture()
    {
    }
};

}//namespace::{anonymous}

BOOST_GLOBAL_FIXTURE(GlobalFixture);

==> fred-backend-registry_2.2.0.orig.tar.gz: fred-backend-registry-2.2.0/test/main.cc <==
/*
 * Copyright (C) 2018-2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE TestBackendRegistry

#include "config.h"

#include "test/setup/fixture/cfg.hh"
#include "test/setup/fixture/contact.hh"
#include "test/setup/fixture/registrar.hh"

#include "libfred/db_settings.hh"
#include "libfred/registrable_object/contact/create_contact.hh"
#include "libfred/registrar/get_registrar_handles.hh"

#include "util/db/manager.hh"
#include "util/log/add_log_device.hh"
#include "util/log/logger.hh"

// dynamic library version
#include <boost/test/unit_test.hpp>
#include <boost/assign/list_of.hpp>

#include <cstdlib>
#include <iostream>

namespace {

enum class LogHandler
{
    is_set,
    is_not_set
};

class SetLogHandler : public boost::static_visitor<LogHandler>
{
public:
    explicit SetLogHandler(Logging::Log& logger_instance) : logger_instance_(logger_instance) { }
    LogHandler operator()(const boost::blank&)const
    {
        return LogHandler::is_not_set;
    }
    LogHandler operator()(const Test::Setup::Fixture::Cfg::Options::Console& option)const
    {
        Logging::add_console_device(logger_instance_, option.min_severity);
        return LogHandler::is_set;
    }
    LogHandler operator()(const Test::Setup::Fixture::Cfg::Options::LogFile& option)const
    {
        Logging::add_file_device(logger_instance_, option.file_name, option.min_severity);
        return LogHandler::is_set;
    }
    LogHandler operator()(const Test::Setup::Fixture::Cfg::Options::SysLog& option)const
    {
        Logging::add_syslog_device(logger_instance_, option.facility, option.min_severity);
        return LogHandler::is_set;
    }
private:
    Logging::Log& logger_instance_;
};

std::string make_connect_string(const Test::Setup::Fixture::Cfg::Options::Database& option)
{
    std::string result;
    if (option.host != boost::none)
    {
        result = "host=" + *option.host;
    }
    if (option.port != boost::none)
    {
        if (!result.empty())
        {
            result += " ";
        }
        result += "port=" + std::to_string(*option.port);
    }
    if (option.user != boost::none)
    {
        if (!result.empty())
        {
            result += " ";
        }
        result += "user=" + *option.user;
    }
    if (option.dbname != boost::none)
    {
        if (!result.empty())
        {
            result += " ";
        }
        result += "dbname=" + *option.dbname;
    }
    if (option.password != boost::none)
    {
        if (!result.empty())
        {
            result += " ";
        }
        result += "password=" + *option.password;
    }
    return result;
}

struct GlobalConfigFixture
{
    GlobalConfigFixture()
    {
        try
        {
            Test::Setup::Fixture::Cfg::Options::init(boost::unit_test::framework::master_test_suite().argc,
                                                     boost::unit_test::framework::master_test_suite().argv);
        }
        catch (const Test::Setup::Fixture::Cfg::HelpRequested& help)
        {
            std::cout << help.what() << std::endl;
            std::exit(EXIT_SUCCESS);
        }
        catch (const Test::Setup::Fixture::Cfg::UnknownOption& unknown_option)
        {
            std::cerr << unknown_option.what() << std::endl;
            std::exit(EXIT_FAILURE);
        }
        catch (const Test::Setup::Fixture::Cfg::MissingOption& missing_option)
        {
            std::cerr << missing_option.what() << std::endl;
            std::exit(EXIT_FAILURE);
        }
        catch (const Test::Setup::Fixture::Cfg::Exception& e)
        {
            std::cerr << "Configuration problem: " << e.what() << std::endl;
            std::exit(EXIT_FAILURE);
        }
        catch (const std::exception& e)
        {
            std::cerr << "Exception caught: " << e.what() << std::endl;
            std::exit(EXIT_FAILURE);
        }
        catch (...)
        {
            std::cerr << "Unexpected exception caught" << std::endl;
            std::exit(EXIT_FAILURE);
        }

        switch (boost::apply_visitor(SetLogHandler(LOGGER), Test::Setup::Fixture::Cfg::Options::get().log))
        {
            case LogHandler::is_set:
                LOGGER.debug("test-fred-registry-services configuration successfully loaded");
                break;
            case LogHandler::is_not_set:
                std::cout << "test-fred-registry-services configuration successfully loaded" << std::endl;
                break;
        }
    }
};

class GlobalAdminDatabaseFixture : public GlobalConfigFixture
{
public:
    GlobalAdminDatabaseFixture()
        : original_db_name_(check_dbname(*Test::Setup::Fixture::Cfg::Options::get().database.dbname)),
          temporary_copy_db_name_(check_dbname(original_db_name_ + "_tmp_copy")),
          result_db_name_(check_dbname(original_db_name_ + "_test_result")),
          db_connect_string_(make_connect_string(Test::Setup::Fixture::Cfg::Options::get().admin_database)),
          db_user_(*Test::Setup::Fixture::Cfg::Options::get().database.user)
    {
        LOGGER.debug("GlobalAdminDatabaseFixture()"
                ":original_db_name_(" + original_db_name_ + "),"
                 "temporary_copy_db_name_(" + temporary_copy_db_name_ + "),"
                 "result_db_name_(" + result_db_name_ + "),"
                 "db_connect_string_(" + db_connect_string_ + "),"
                 "db_user_(" + db_user_ + ")");
        const auto db_conn = this->get_admin_connection();
        force_copy_db(db_conn.get(), original_db_name_, temporary_copy_db_name_, db_user_);
    }
    ~GlobalAdminDatabaseFixture()
    {
        try
        {
            const auto db_conn = this->get_admin_connection();
            noexcept_force_copy_db(db_conn.get(), original_db_name_, result_db_name_, db_user_);
            noexcept_force_copy_db(db_conn.get(), temporary_copy_db_name_, original_db_name_, db_user_);
            noexcept_force_drop_db(db_conn.get(), temporary_copy_db_name_, db_user_);
            noexcept_enable_connections(db_conn.get(), result_db_name_);
            noexcept_enable_connections(db_conn.get(), original_db_name_);
            noexcept_enable_connections(db_conn.get(), temporary_copy_db_name_);
        }
        catch (...)
        {
            try
            {
                LOGGER.error("get_admin_connection() failed");
            }
            catch (...)
            {
            }
        }
    }
private:
    std::unique_ptr<Database::StandaloneConnection> get_admin_connection()const
    {
        return Database::StandaloneManager(db_connect_string_).acquire();
    }
    static void disable_connections_add_terminate_persistent_connections(
            Database::StandaloneConnection* const conn,
            const std::string& db_name,
            const std::string& db_user)
    {
        conn->exec_params("UPDATE pg_database SET datallowconn=false WHERE datname=$1::TEXT",
                          Database::query_param_list(db_name));

        std::string pid_column_name;
        {
            const Database::Result dbres = conn->exec(
                    "SELECT column_name "
                    "FROM information_schema.columns "
                    "WHERE table_name='pg_stat_activity' AND "
                          "column_name IN ('procpid','pid')");
            if (dbres.size() <= 0)
            {
                std::runtime_error("table 'pg_stat_activity' contains neither procpid nor pid column");
            }
            pid_column_name = static_cast<std::string>(dbres[0][0]);
        }
        const std::string query =
                "SELECT pg_terminate_backend(" + pid_column_name + ") "
                "FROM pg_stat_activity "
                "WHERE (datname=$1::TEXT OR usename=$2::TEXT) AND " + pid_column_name + "!=pg_backend_pid()";
        const auto params = Database::query_param_list(db_name)(db_user);
        while (conn->exec_params(query, params).size() != 0)
        { }
    }
    static void noexcept_enable_connections(
            Database::StandaloneConnection* const conn,
            const std::string& db_name)noexcept
    {
        try
        {
            conn->exec_params("UPDATE pg_database SET datallowconn=true WHERE datname=$1::TEXT",
                              Database::query_param_list(db_name));
        }
        catch (...)
        {
            try
            {
                LOGGER.error("UPDATE pg_database failed");
            }
            catch (...)
            {
            }
        }
    }
    static const std::string& check_dbname(const std::string& db_name)
    {
        static constexpr unsigned max_postgresql_database_name_length = 63;
        if (max_postgresql_database_name_length < db_name.length())
        {
            throw std::runtime_error(
                    "db_name.length(): " + std::to_string(db_name.length()) + " > "
                    "max_postgresql_database_name_length: " + std::to_string(max_postgresql_database_name_length) + " "
                    "db_name: " + db_name);
        }
        return db_name;
    }
    static void force_drop_db(
            Database::StandaloneConnection* const conn,
            const std::string& db_name,
            const std::string& db_user)
    {
        disable_connections_add_terminate_persistent_connections(conn, db_name, db_user);
        conn->exec("DROP DATABASE IF EXISTS \"" + db_name + "\"");
    }
    static void force_copy_db(
            Database::StandaloneConnection* const conn,
            const std::string& src_name,
            const std::string& dst_name,
            const std::string& db_user)
    {
        force_drop_db(conn, dst_name, db_user);
        conn->exec("CREATE DATABASE \"" + dst_name + "\" TEMPLATE \"" + src_name + "\"");
    }
    static void noexcept_force_drop_db(
            Database::StandaloneConnection* const conn,
            const std::string& db_name,
            const std::string& db_user)noexcept
    {
        try
        {
            disable_connections_add_terminate_persistent_connections(conn, db_name, db_user);
        }
        catch (...)
        {
            try
            {
                LOGGER.error("disable_connections_add_terminate_persistent_connections failed");
            }
            catch (...)
            {
            }
        }
        try
        {
            conn->exec("DROP DATABASE IF EXISTS \"" + db_name + "\"");
        }
        catch (...)
        {
            try
            {
                LOGGER.error("DROP DATABASE failed");
            }
            catch (...)
            {
            }
        }
    }
    static void noexcept_force_copy_db(
            Database::StandaloneConnection* const conn,
            const std::string& src_name,
            const std::string& dst_name,
            const std::string& db_user)
    {
        noexcept_force_drop_db(conn, dst_name, db_user);
        try
        {
            conn->exec("CREATE DATABASE \"" + dst_name + "\" TEMPLATE \"" + src_name + "\"");
        }
        catch (...)
        {
            try
            {
                LOGGER.error("CREATE DATABASE failed");
            }
            catch (...)
            {
            }
        }
    }
    const std::string original_db_name_;
    const std::string temporary_copy_db_name_;
    const std::string result_db_name_;
    const std::string db_connect_string_;
    const std::string db_user_;
};

struct GlobalDatabaseFixture : GlobalAdminDatabaseFixture
{
    GlobalDatabaseFixture()
    {
        Database::emplace_default_manager<Database::StandaloneManager>(
                make_connect_string(Test::Setup::Fixture::Cfg::Options::get().database));
    }
};

struct GlobalRegistryDataFixture : GlobalDatabaseFixture
{
    GlobalRegistryDataFixture()
    {
        LibFred::OperationContextCreator ctx;
        Test::Registrar::init<Test::Registrar::system>(ctx);
        LibFred::Contact::PlaceAddress place;
        place.street1 = "Kontaktní 1";
        place.city = "Praha 1";
        place.postalcode = "111 50";
        place.country = "CZ";
        LibFred::ContactAddressList addresses_by_type;
        LibFred::ContactAddress address;
        address.street1 = "Korespondenční 1";
        address.city = "Praha 1";
        address.postalcode = "111 50";
        address.country = "CZ";
        addresses_by_type[LibFred::ContactAddressType::MAILING] = address;
        address.street1 = "Fakturační 1";
        addresses_by_type[LibFred::ContactAddressType::BILLING] = address;
        address.street1 = "Doručovací 1";
        addresses_by_type[LibFred::ContactAddressType::SHIPPING] = address;
        address.street1 = "Doručovací 2";
        addresses_by_type[LibFred::ContactAddressType::SHIPPING_2] = address;
        address.street1 = "Doručovací 3";
        addresses_by_type[LibFred::ContactAddressType::SHIPPING_3] = address;
        Test::Contact::init(
                ctx,
                Test::Contact::make_handle("TEST_KONTAKT_0001"),
                Test::Registrar::get<Test::Registrar::system>().get_handle(),
                Test::Contact::make_name("Test Kontakt"),
                Test::Contact::make_organization("První Testovací a.s."),
                Test::Contact::make_authinfo("AbCdEfGhIjK1"),
                Test::Contact::make_place_address(place),
                Test::Contact::make_contact_address_list(addresses_by_type),
                Test::Contact::Disclose::name(),
                Test::Contact::Disclose::organization(),
                Test::Contact::Disclose::address(),
                Test::Contact::Hide::telephone(),
                Test::Contact::Hide::fax(),
                Test::Contact::Hide::email(),
                Test::Contact::Hide::notify_email(),
                Test::Contact::Hide::vat_identification_number(),
                Test::Contact::Hide::ident());
        ctx.commit_transaction();
    }
};

struct GlobalFixture : GlobalRegistryDataFixture
{
    GlobalFixture()
    {
    }
};

}//namespace::{anonymous}

BOOST_GLOBAL_FIXTURE(GlobalFixture);
